// VideoRender.cpp: implementation of the CVideoRender class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ProbeDemo.h"
#include "VideoRender.h"
#include <process.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
UINT WINAPI _RenderThread(LPVOID lpParam)
{
	CVideoRender *pRender = (CVideoRender *)lpParam;
	if(pRender)
	{
		pRender->ExecuteThread();
	}
	return 0;
}
CVideoRender::CVideoRender()
{
	m_Dx3D		= NULL;
	m_d3dDevice = NULL;
	m_pStaticTextBuffer = NULL;

	m_bStop	= FALSE;
	m_hThread = NULL;
	m_hEventThread = CreateEvent(NULL, TRUE, FALSE, NULL);
	m_Texture	= new CTexture();
}

CVideoRender::~CVideoRender()
{
	if(!m_bStop)
	{
		m_bStop = TRUE;
		SetEvent(m_hEventThread);
	}
	if(m_hThread)
	{
		if(WaitForSingleObject(m_hThread,5000) == WAIT_TIMEOUT)
		{
			DWORD ExitCode=0;
    		if(::GetExitCodeThread(m_hThread,&ExitCode))
    		{
    			TerminateThread(m_hThread,ExitCode);
    		}	
		}	
		m_hThread=NULL;
	}

	Clear();

	if(m_hEventThread)
	{
		CloseHandle(m_hEventThread);
		m_hEventThread = NULL;
	}

    if ( m_Texture )
    {
        delete m_Texture ;
        m_Texture = NULL ;
    }
}
void CVideoRender::ExecuteThread()
{
	while(!m_bStop)
	{
		WaitForSingleObject(m_hEventThread, 20);
		if(m_bStop) break;
	
		Render();
	}
}
HRESULT CVideoRender::Init(HWND hWnd, int x, int y, DWORD dwWWidth, DWORD dwWHeight)
{
	// ̽ 
	m_Dx3D = Direct3DCreate8(D3D_SDK_VERSION);

	D3DDISPLAYMODE CurrentMode;	
	if( FAILED( m_Dx3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &CurrentMode ) ) )
	{
		return E_FAIL;
	}

	D3DPRESENT_PARAMETERS d3dpp; 
	ZeroMemory( &d3dpp, sizeof(d3dpp) );
	d3dpp.Windowed			= TRUE;
	d3dpp.SwapEffect		= D3DSWAPEFFECT_DISCARD;
	d3dpp.BackBufferFormat	= CurrentMode.Format;
	d3dpp.EnableAutoDepthStencil = TRUE;
	d3dpp.AutoDepthStencilFormat = D3DFMT_D16;

//	d3dpp.hDeviceWindow		= hWnd;
//	d3dpp.BackBufferWidth	= dwWWidth;
//	d3dpp.BackBufferHeight	= dwWHeight;
//	d3dpp.BackBufferCount	= 1;

	//d3dpp.AdapterOrdinal = D3DADAPTER_DEFAULT;
	//d3dpp.DeviceType     = DeviceType;
	//d3dpp.hFocusWindow   = WindowHandle;
	//d3dpp.BehaviorFlags  = Behavior;

	if( FAILED( m_Dx3D->CreateDevice( D3DADAPTER_DEFAULT, 
										D3DDEVTYPE_HAL, 
										hWnd,
										D3DCREATE_SOFTWARE_VERTEXPROCESSING,
										&d3dpp, 
										&m_d3dDevice ) ) )
	{
		return E_FAIL;
	}

    // Turn off culling
    m_d3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );

    // Turn off D3D lighting
    m_d3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );

    // Turn on the zbuffer
    m_d3dDevice->SetRenderState( D3DRS_ZENABLE, FALSE );

	// ׽Ʈ  İ = TextureAlpha * TextureFactorAlpha
	// ؽۿ  Ƿ TextureFactor Ѵ.
	m_d3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_MODULATE );
	m_d3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TFACTOR );
	m_d3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG2, D3DTA_TEXTURE );

	//    ׽Ʈ. (İ 0̸ ó)
	m_d3dDevice->SetRenderState( D3DRS_ALPHATESTENABLE, TRUE );
	m_d3dDevice->SetRenderState( D3DRS_ALPHAFUNC, D3DCMP_NOTEQUAL );
	m_d3dDevice->SetRenderState( D3DRS_ALPHAREF, 0 );

	//   ON !
//	m_d3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
	// ĺ Լ  .
//	m_d3dDevice->SetRenderState( D3DRS_SRCBLEND  , D3DBLEND_SRCALPHA );
//	m_d3dDevice->SetRenderState( D3DRS_DESTBLEND , D3DBLEND_INVSRCALPHA );

	// ÷ ̵ = Texture * TextureFactor
	// ؽۿ  Ƿ TextureFactor Ѵ.
//	m_d3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP, D3DTOP_MODULATE );
//	m_d3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TFACTOR );
//	m_d3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_TEXTURE );

	// ͸ off
	// Ȯ ɰ쿣 ƹ ͸ Ѵ°  .
	// ׷ Ʈ  ͸ Ͼ⶧
	//  κ ׵θ ִ.
	// ͸  2DƮ ....ϰ Ÿ.
//	m_d3dDevice->SetTextureStageState( 0, D3DTSS_MAGFILTER, D3DTEXF_POINT );
//    m_d3dDevice->SetTextureStageState( 0, D3DTSS_MINFILTER, D3DTEXF_POINT );

	m_Texture->Init(m_d3dDevice, dwWWidth, dwWHeight, CurrentMode.Format);
	//m_Texture.Init(m_d3dDevice, dwWWidth, dwWHeight, D3DFMT_X8R8G8B8);

	m_Font.CreateFont(m_d3dDevice, "Arial", 20);
	m_pStaticTextBuffer = m_Font.CreateStaticText(5.0f, 5.0f, "test", 0xff00ff00);

	if(!m_hThread)
	{
		DWORD dwThreadID;
		m_hThread = (HANDLE)_beginthreadex(NULL, 0, _RenderThread, this, 0, (unsigned *)&dwThreadID);
	}

	return S_OK;
}
void CVideoRender::Clear()
{
	if(m_d3dDevice)
	{
		m_d3dDevice->Release();
		m_d3dDevice = NULL;
	}
	if(m_Dx3D)
	{
		m_Dx3D->Release();
		m_Dx3D = NULL;
	}
	if(m_pStaticTextBuffer)
	{
		m_Font.DestroyStaticText(m_pStaticTextBuffer);
	}
	m_Font.DestroyFont();
}

BOOL CVideoRender::Start()
{
	return TRUE;
}
void CVideoRender::Stop()
{
	
}
void CVideoRender::Render()
{
	PreRender();

	m_Texture->Draw(0,0,m_Texture->GetWidth(),m_Texture->GetHeight(), 0,0,1,1, D3DCOLOR_XRGB(255,255,255));

	/*
	m_Texture[1].Draw(0,0,m_Texture[1].GetWidth(),m_Texture[1].GetHeight(),
				      0.5,0.5,1,1, D3DCOLOR_XRGB(255,255,255));
	m_Texture[2].Draw(0,0,m_Texture[2].GetWidth(),m_Texture[2].GetHeight(),
				      0,0,1,1, D3DCOLOR_XRGB(255,255,255));
	m_Texture[3].Draw(0,0,m_Texture[3].GetWidth(),m_Texture[3].GetHeight(),
				      0.5,0,1,1, D3DCOLOR_XRGB(255,255,255));
					  */
//	m_d3dDevice->SetTexture(0, m_Texture.GetTexture()); 
//	m_d3dDevice->SetStreamSource(0, m_Texture.GetVertexBuffer(), sizeof(CUSTOMVERTEX)); 
//	m_d3dDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP , 0, 2);
	m_Font.DrawStaticText(m_pStaticTextBuffer, 0, 4);

	PostRender();
}
void CVideoRender::DrawImage(BYTE *lpData, DWORD dwWidth, DWORD dwHeight)
{
	m_Texture->Blt(lpData, dwWidth, dwHeight, 4*dwWidth);
}

void CVideoRender::PreRender()
{
	//Clear the device
	m_d3dDevice->Clear(0, NULL,	D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER,	D3DCOLOR_XRGB(0, 0, 0), 1.0f, 0);

	//Call BeginScene to set up the device
	m_d3dDevice->BeginScene();
	
	return;
}

HRESULT CVideoRender::PostRender()
{
	// End the scene
	m_d3dDevice->EndScene();
	
	return m_d3dDevice->Present(NULL, NULL, NULL, NULL);
}