// SoundCaptureManager.cpp: implementation of the CSoundCaptureManager class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SoundCaptureManager.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
UINT WINAPI SoundCaptureThread(LPVOID lpParam)
{
	CSoundCaptureManager * pCaptureManager = (CSoundCaptureManager *)lpParam;
	pCaptureManager->ExecuteThread();
	return 0;
}

CSoundCaptureManager::CSoundCaptureManager()
{
	m_bActive   = FALSE;
	CB_CALLBACK = NULL;
	m_lpCallbackData = NULL;

	m_pDSCapture         = NULL;
	m_pDSBCapture        = NULL;
	m_pDSNotify          = NULL;

	m_guidCaptureDevice  = DSDEVID_DefaultCapture;
	ZeroMemory(&m_wfxInput, sizeof(m_wfxInput));
	m_aPosNotify[ NUM_REC_NOTIFICATIONS + 1 ];  
	m_hNotificationEvent[0]	= CreateEvent( NULL, FALSE, FALSE, NULL );
	m_hNotificationEvent[1]	= CreateEvent( NULL, FALSE, FALSE, NULL );
	m_dwCaptureBufferSize   = 0;
	m_dwNextCaptureOffset   = 0;
	m_dwNotifySize			= 0;	
}

CSoundCaptureManager::~CSoundCaptureManager()
{
	StopCapture();

	Finalize();

	if(m_hNotificationEvent[0])
	{
		CloseHandle(m_hNotificationEvent[0]);
		m_hNotificationEvent[0] = INVALID_HANDLE_VALUE;
	}
	if(m_hNotificationEvent[1])
	{
		CloseHandle(m_hNotificationEvent[1]);
		m_hNotificationEvent[1] = INVALID_HANDLE_VALUE;
	}
}

BOOL CSoundCaptureManager::Initialize(LPFNC_CAPTUREDATA Function_Callback, LPVOID lpCallbackData, DWORD dwPrimaryChannels, DWORD dwPrimaryFreq, DWORD dwPrimaryBitRate)
{
	CB_CALLBACK = Function_Callback;
	m_lpCallbackData = lpCallbackData;

    if( S_OK != InitDirectSound() )
	{
		return FALSE;
	}

	SetBufferFormat(dwPrimaryChannels,dwPrimaryFreq,dwPrimaryBitRate);

    return TRUE;
}

void CSoundCaptureManager::Finalize()
{
    // Release DirectSound interfaces
	if(m_pDSNotify)
	{
		m_pDSNotify->Release();
		m_pDSNotify = NULL;
	}
	if(m_pDSBCapture)
	{
		m_pDSBCapture->Release();
		m_pDSBCapture = NULL;
	}
	if(m_pDSCapture)
	{
		m_pDSCapture->Release();
		m_pDSCapture = NULL;
	}

    // Release COM
    CoUninitialize();
}

HRESULT CSoundCaptureManager::InitDirectSound()
{
    HRESULT hr;

    ZeroMemory( &m_aPosNotify, sizeof(DSBPOSITIONNOTIFY) * 
                               (NUM_REC_NOTIFICATIONS + 1) );
    m_dwCaptureBufferSize = 0;
    m_dwNotifySize        = 0;

    // Initialize COM
    if( FAILED( hr = CoInitialize(NULL) ) )
        return DXTRACE_ERR( TEXT("CoInitialize"), hr );


    // Create IDirectSoundCapture using the preferred capture device
    if( FAILED( hr = DirectSoundCaptureCreate( &m_guidCaptureDevice, &m_pDSCapture, NULL ) ) )
        return DXTRACE_ERR( TEXT("DirectSoundCaptureCreate"), hr );

    return S_OK;
}

void CSoundCaptureManager::SetBufferFormat(DWORD dwPrimaryChannels, DWORD dwPrimaryFreq, DWORD dwPrimaryBitRate)
{
    ZeroMemory( &m_wfxInput, sizeof(m_wfxInput));
    m_wfxInput.wFormatTag = WAVE_FORMAT_PCM; 
    m_wfxInput.nChannels       = (WORD) dwPrimaryChannels; 
    m_wfxInput.nSamplesPerSec  = dwPrimaryFreq; 
    m_wfxInput.wBitsPerSample  = (WORD) dwPrimaryBitRate; 
    m_wfxInput.nBlockAlign     = m_wfxInput.wBitsPerSample / 8 * m_wfxInput.nChannels;
    m_wfxInput.nAvgBytesPerSec = m_wfxInput.nSamplesPerSec * m_wfxInput.nBlockAlign;
}

HRESULT CSoundCaptureManager::InitNotifications()
{
    HRESULT hr; 

    if( NULL == m_pDSBCapture )
        return E_FAIL;

    // Create a notification event, for when the sound stops playing
    if( FAILED( hr = m_pDSBCapture->QueryInterface( IID_IDirectSoundNotify, (VOID**)&m_pDSNotify ) ) )
        return DXTRACE_ERR( TEXT("QueryInterface"), hr );

    // Setup the notification positions
    for( INT i = 0; i < NUM_REC_NOTIFICATIONS; i++ )
    {
        m_aPosNotify[i].dwOffset = (m_dwNotifySize * i) + m_dwNotifySize - 1;
        m_aPosNotify[i].hEventNotify = m_hNotificationEvent[0];             
    }
    m_aPosNotify[i].dwOffset     = DSBPN_OFFSETSTOP;
    m_aPosNotify[i].hEventNotify = m_hNotificationEvent[1];
    
    // Tell DirectSound when to notify us. the notification will come in the from 
    // of signaled events that are handled in WinMain()
    if( FAILED( hr = m_pDSNotify->SetNotificationPositions( NUM_REC_NOTIFICATIONS, m_aPosNotify ) ) )
        return DXTRACE_ERR( TEXT("SetNotificationPositions"), hr );

    return S_OK;
}

void CSoundCaptureManager::ExecuteThread()
{
    while( m_bActive) 
    { 
		DWORD dwResult = MsgWaitForMultipleObjects( 2, m_hNotificationEvent,FALSE, INFINITE, QS_ALLEVENTS );
        switch( dwResult )
        {
            case WAIT_OBJECT_0 + 0:
                // g_hNotificationEvents[0] is signaled

                // This means that DirectSound just finished playing 
                // a piece of the buffer, so we need to fill the circular 
                // buffer with new sound from the wav file

 				if(!m_bActive) break;

				if( S_OK != CapturedData() )
				{
				}
                break;

            case WAIT_OBJECT_0 + 1:
                // g_hNotificationEvents[1] is signaled

                // This means that the stop event was signaled, so 
                // any needed update could be done here
				if(!m_bActive) break;
                break;

            case WAIT_OBJECT_0 + 2:
                // Windows messages are available
                /*
				while( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) ) 
                { 
                    TranslateMessage( &msg ); 
                    DispatchMessage( &msg );
                }
				*/
                break;
        }											  
		//Sleep(1);
    }
}

BOOL CSoundCaptureManager::StartCapture()
{
    HRESULT hr;

	if(!m_bActive)
	{
		// Create a capture buffer, and tell the capture 
		// buffer to start recording   
		if( FAILED( hr = CreateCaptureBuffer( &m_wfxInput ) ) )
			return FALSE;

		if( FAILED( hr = m_pDSBCapture->Start( DSCBSTART_LOOPING ) ) )
			return FALSE;

		m_bActive = TRUE;
		DWORD  dwThreadID;
		m_hThread = (HANDLE)_beginthreadex(NULL, 0, SoundCaptureThread, this, 0, (unsigned *)&dwThreadID);
	}
	return TRUE;
}

void CSoundCaptureManager::StopCapture()
{
	HRESULT hr;

	if(m_bActive)
	{
		m_bActive = FALSE;
		SetEvent(m_hNotificationEvent[1]);

		if(m_hThread)
		{
			if(WaitForSingleObject(m_hThread,5000) == WAIT_TIMEOUT)
			{
				DWORD ExitCode=0;
    			if(::GetExitCodeThread(m_hThread,&ExitCode))
    			{
    				TerminateThread(m_hThread,ExitCode);
    			}	
			}
			m_hThread=NULL;
		}

		// Stop the capture and read any data that 
		// was not caught by a notification
		if( NULL == m_pDSBCapture )
			return;

		// Stop the buffer, and read any data that was not 
		// caught by a notification
		if( FAILED( hr = m_pDSBCapture->Stop() ) )
			return;

		if( FAILED( hr = CapturedData() ) )
			return;
	}
	return;
}



HRESULT CSoundCaptureManager::CapturedData()
{
    HRESULT hr;
    VOID*   pbCaptureData    = NULL;
    DWORD   dwCaptureLength;
    VOID*   pbCaptureData2   = NULL;
    DWORD   dwCaptureLength2;
    VOID*   pbPlayData       = NULL;
    DWORD   dwReadPos;
    DWORD   dwCapturePos;
    LONG lLockSize;

    if( NULL == m_pDSBCapture )
        return S_FALSE;

    if( FAILED( hr = m_pDSBCapture->GetCurrentPosition( &dwCapturePos, &dwReadPos ) ) )
        return DXTRACE_ERR( TEXT("GetCurrentPosition"), hr );

    lLockSize = dwReadPos - m_dwNextCaptureOffset;
    if( lLockSize < 0 )
        lLockSize += m_dwCaptureBufferSize;

    // Block align lock size so that we are always write on a boundary
    lLockSize -= (lLockSize % m_dwNotifySize);

    if( lLockSize == 0 )
        return S_FALSE;

    // Lock the capture buffer down
    if( FAILED( hr = m_pDSBCapture->Lock( m_dwNextCaptureOffset, lLockSize, 
                                          &pbCaptureData, &dwCaptureLength, 
                                          &pbCaptureData2, &dwCaptureLength2, 0L ) ) )
        return DXTRACE_ERR( TEXT("Lock"), hr );


	DWORD dwSize = dwCaptureLength + dwCaptureLength2;
	BYTE* pBuf = new BYTE[dwSize];

	memcpy(pBuf , (BYTE*)pbCaptureData, dwCaptureLength);
	//if(CB_CALLBACK)
	//{
	//	CB_CALLBACK((BYTE *)pbCaptureData, dwCaptureLength, m_lpCallbackData);
	//}

    // Move the capture offset along
    m_dwNextCaptureOffset += dwCaptureLength; 
    m_dwNextCaptureOffset %= m_dwCaptureBufferSize; // Circular buffer

    if( pbCaptureData2 != NULL )
    {
		//if(CB_CALLBACK)
		//{
		//	CB_CALLBACK((BYTE *)pbCaptureData2, dwCaptureLength2, m_lpCallbackData);
		//}

        // Move the capture offset along
        m_dwNextCaptureOffset += dwCaptureLength2; 
        m_dwNextCaptureOffset %= m_dwCaptureBufferSize; // Circular buffer

		memcpy(pBuf+dwCaptureLength , (BYTE*)pbCaptureData2, dwCaptureLength2);
    }

    // Unlock the capture buffer
    m_pDSBCapture->Unlock( pbCaptureData,  dwCaptureLength, 
                           pbCaptureData2, dwCaptureLength2 );


	if(CB_CALLBACK)
	{
		CB_CALLBACK(pBuf, dwSize, m_lpCallbackData);
	}
	delete pBuf;

    return S_OK;
}

HRESULT CSoundCaptureManager::CreateCaptureBuffer(WAVEFORMATEX *pwfxInput)
{
    HRESULT hr;
    DSCBUFFERDESC dscbd;

	if(m_pDSNotify)
	{
		m_pDSNotify->Release();
		m_pDSNotify = NULL;
	}
	if(m_pDSBCapture)
	{
		m_pDSBCapture->Release();
		m_pDSBCapture = NULL;
	}

    // Set the notification size
    m_dwNotifySize = MAX( 1024, m_wfxInput.nAvgBytesPerSec / 8 ); 
	//m_dwNotifySize = m_wfxInput.nAvgBytesPerSec / 8;
    m_dwNotifySize -= m_dwNotifySize % m_wfxInput.nBlockAlign; 
	//m_dwNotifySize = 960;  // g.723.1  ó

	// 30 ms  nitification ޱ 
	//m_dwNotifySize = m_wfxInput.nAvgBytesPerSec / 16;   
	//m_dwNotifySize -= m_dwNotifySize % m_wfxInput.nBlockAlign; 

    // Set the buffer sizes 
    m_dwCaptureBufferSize = m_dwNotifySize * NUM_REC_NOTIFICATIONS;

	if(m_pDSNotify)
	{
		m_pDSNotify->Release();
		m_pDSNotify = NULL;
	}
	if(m_pDSBCapture)
	{
		m_pDSBCapture->Release();
		m_pDSBCapture = NULL;
	}

    // Create the capture buffer
    ZeroMemory( &dscbd, sizeof(dscbd) );
    dscbd.dwSize        = sizeof(dscbd);
    dscbd.dwBufferBytes = m_dwCaptureBufferSize;
    dscbd.lpwfxFormat   = &m_wfxInput; // Set the format during creatation

    if( FAILED( hr = m_pDSCapture->CreateCaptureBuffer( &dscbd, 
                                                        &m_pDSBCapture, 
                                                        NULL ) ) )
        return DXTRACE_ERR( TEXT("CreateCaptureBuffer"), hr );

    m_dwNextCaptureOffset = 0;

    if( FAILED( InitNotifications() ) )
        return hr;

    return S_OK;
}
